from enum import Enum
from datetime import datetime
from typing import Optional, List


class ProcessData:
    ID_object: int
    ID_polygon: int
    title: str
    type: None
    icon: str
    mode: int
    created_at: datetime
    updated_at: datetime
    ID_object_process_read: int
    ID_process_write: int
    ID_process_read: int

    def __init__(self, ID_object: int, ID_polygon: int, title: str, type: None, icon: str, mode: int,
                 created_at: datetime, updated_at: datetime, ID_object_process_read: int, ID_process_write: int,
                 ID_process_read: int) -> None:
        self.ID_object = ID_object
        self.ID_polygon = ID_polygon
        self.title = title
        self.type = type
        self.icon = icon
        self.mode = mode
        self.created_at = created_at
        self.updated_at = updated_at
        self.ID_object_process_read = ID_object_process_read
        self.ID_process_write = ID_process_write
        self.ID_process_read = ID_process_read


class AnalogType(Enum):
    INPUT = "input"


class AnalogData:
    ID_analog_math: int
    ID_analog: int
    type: AnalogType
    math: str
    created_at: datetime
    updated_at: datetime
    ID_aux: int
    max_voltage: Optional[int]
    max_value: Optional[int]

    def __init__(self, ID_analog_math: int, ID_analog: int, type: AnalogType, math: str, created_at: datetime,
                 updated_at: datetime, ID_aux: int, max_voltage: Optional[int], max_value: Optional[int]) -> None:
        self.ID_analog_math = ID_analog_math
        self.ID_analog = ID_analog
        self.type = type
        self.math = math
        self.created_at = created_at
        self.updated_at = updated_at
        self.ID_aux = ID_aux
        self.max_voltage = max_voltage
        self.max_value = max_value


class ExpansionData:
    ID_AUX_station: int
    ID_station: int
    title: str
    output: int
    type: int
    expansion: int
    local: int
    settings: None
    created_at: datetime
    updated_at: datetime
    ID_aux_expansion: int
    ID_expansion_version: int
    ID_expansion: int
    pan: int
    address: int

    def __init__(self, ID_AUX_station: int, ID_station: int, title: str, output: int, type: int, expansion: int,
                 local: int, settings: None, created_at: datetime, updated_at: datetime, ID_aux_expansion: int,
                 ID_expansion_version: int, ID_expansion: int, pan: int, address: int) -> None:
        self.ID_AUX_station = ID_AUX_station
        self.ID_station = ID_station
        self.title = title
        self.output = output
        self.type = type
        self.expansion = expansion
        self.local = local
        self.settings = settings
        self.created_at = created_at
        self.updated_at = updated_at
        self.ID_aux_expansion = ID_aux_expansion
        self.ID_expansion_version = ID_expansion_version
        self.ID_expansion = ID_expansion
        self.pan = pan
        self.address = address


class ReaderData:
    ID_object_reader: int
    ID_object: int
    created_at: datetime
    updated_at: datetime
    ID_polygon: int
    title: str
    type: int
    icon: str
    mode: int
    ID_aux_station: int
    ID_AUX_station: int
    ID_station: int
    output: int
    expansion: int
    local: int
    settings: Optional[str]
    ID_org: int
    ID_station_type: int
    serial: int
    code: str
    active: int
    token: int
    location: str
    number: str
    Expansion: Optional[List[ExpansionData]] = []
    Analog: Optional[List[AnalogData]] = []

    def __init__(self, ID_object_reader: int, ID_object: int, created_at: datetime,
                 updated_at: datetime, ID_polygon: int, title: str, type: int, icon: str, mode: int,
                 ID_aux_station: int, ID_AUX_station: int, ID_station: int, output: int, local: int,
                 settings: Optional[str], ID_org: int, ID_station_type: int, expansion: int, serial: int, code: str, active: int,
                 token: int, location: str, number: str, Expansion: Optional[List[ExpansionData]] = None,
                 Analog: Optional[List[AnalogData]] = None) -> None:
        self.ID_object_reader = ID_object_reader
        self.ID_object = ID_object
        self.created_at = created_at
        self.updated_at = updated_at
        self.ID_polygon = ID_polygon
        self.title = title
        self.type = type
        self.icon = icon
        self.mode = mode
        self.ID_aux_station = ID_aux_station
        self.ID_AUX_station = ID_AUX_station
        self.ID_station = ID_station
        self.output = output
        self.local = local
        self.settings = settings
        self.ID_org = ID_org
        self.ID_station_type = ID_station_type
        self.serial = serial
        self.code = code
        self.active = active
        self.token = token
        self.location = location
        self.number = number
        self.expansion = expansion
        self.Expansion = []
        self.Analog = []
        if Expansion:
            for param in Expansion:
                self.Expansion.append(ExpansionData(**dict(param)))
        if Analog:
            for param in Analog:
                self.Analog.append(AnalogData(**dict(param)))


class Object:
    ID_object: int
    ID_polygon: int
    title: str
    type: str
    icon: str
    mode: int
    created_at: datetime
    updated_at: datetime
    reader: Optional[List[ReaderData]] = []
    process: Optional[List[ProcessData]] = []

    def __init__(self, ID_object: int, ID_polygon: int, title: str, type: str, icon: str, mode: int,
                 created_at: datetime, updated_at: datetime, Reader: Optional[List[ReaderData]] = None,
                 Process: Optional[List[ProcessData]] = None) -> None:
        self.ID_object = ID_object
        self.ID_polygon = ID_polygon
        self.title = title
        self.type = type
        self.icon = icon
        self.mode = mode
        self.created_at = created_at
        self.updated_at = updated_at
        self.reader = []
        self.process = []
        if Reader:
            for param in Reader:
                self.reader.append(ReaderData(**dict(param)))

        if Process:
            for param in Process:
                self.process.append(ProcessData(**dict(param)))


class ObjectData:
    object: List[Object]

    def __init__(self, data) -> None:
        self.data = []
        for param in data:
            self.data.append(Object(**dict(param)))



from datetime import datetime
from typing import List
import re


def convertCamelCase(nameArray):
    data = {}

    for key in nameArray:
        data[re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()] = nameArray[key]

    return data


class Current:
    time: str
    symbol: str
    symbol_phrase: str
    temperature: int
    feels_like_temp: int
    rel_humidity: int
    dew_point: int
    wind_speed: int
    wind_dir_string: str
    wind_gust: int
    precip_prob: int
    precip_rate: None
    cloudiness: int
    thunder_prob: int
    uv_index: int
    pressure: float
    visibility: int

    def __init__(self, time: str, symbol: str, symbol_phrase: str, temperature: int, feels_like_temp: int,
                 rel_humidity: int, dew_point: int, wind_speed: int, wind_dir_string: str, wind_gust: int,
                 precip_prob: int, precip_rate: None, cloudiness: int, thunder_prob: int, uv_index: int,
                 pressure: float, visibility: int) -> None:
        self.time = time
        self.symbol = symbol
        self.symbol_phrase = symbol_phrase
        self.temperature = temperature
        self.feels_like_temp = feels_like_temp
        self.rel_humidity = rel_humidity
        self.dew_point = dew_point
        self.wind_speed = wind_speed
        self.wind_dir_string = wind_dir_string
        self.wind_gust = wind_gust
        self.precip_prob = precip_prob
        self.precip_rate = precip_rate
        self.cloudiness = cloudiness
        self.thunder_prob = thunder_prob
        self.uv_index = uv_index
        self.pressure = pressure
        self.visibility = visibility


class Daily:
    date: datetime
    symbol: str
    symbol_phrase: str
    max_temp: int
    min_temp: int
    max_feels_like_temp: int
    min_feels_like_temp: int
    max_rel_humidity: int
    min_rel_humidity: int
    max_dew_point: int
    min_dew_point: int
    precip_accum: int
    max_wind_speed: int
    wind_dir: int
    max_wind_gust: int
    precip_prob: int
    cloudiness: int
    sunrise: datetime
    sunset: datetime
    sunrise_epoch: int
    sunset_epoch: int
    moonrise: datetime
    moonset: datetime
    moon_phase: int
    uv_index: int
    min_visibility: int
    pressure: float

    def __init__(self, date: datetime, symbol: str, symbol_phrase: str, max_temp: int, min_temp: int,
                 max_feels_like_temp: int, min_feels_like_temp: int, max_rel_humidity: int, min_rel_humidity: int,
                 max_dew_point: int, min_dew_point: int, precip_accum: int, max_wind_speed: int, wind_dir: int,
                 max_wind_gust: int, precip_prob: int, cloudiness: int, sunrise: datetime, sunset: datetime,
                 sunrise_epoch: int, sunset_epoch: int, moonrise: datetime, moonset: datetime, moon_phase: int,
                 uv_index: int, min_visibility: int, pressure: float) -> None:
        self.date = date
        self.symbol = symbol
        self.symbol_phrase = symbol_phrase
        self.max_temp = max_temp
        self.min_temp = min_temp
        self.max_feels_like_temp = max_feels_like_temp
        self.min_feels_like_temp = min_feels_like_temp
        self.max_rel_humidity = max_rel_humidity
        self.min_rel_humidity = min_rel_humidity
        self.max_dew_point = max_dew_point
        self.min_dew_point = min_dew_point
        self.precip_accum = precip_accum
        self.max_wind_speed = max_wind_speed
        self.wind_dir = wind_dir
        self.max_wind_gust = max_wind_gust
        self.precip_prob = precip_prob
        self.cloudiness = cloudiness
        self.sunrise = sunrise
        self.sunset = sunset
        self.sunrise_epoch = sunrise_epoch
        self.sunset_epoch = sunset_epoch
        self.moonrise = moonrise
        self.moonset = moonset
        self.moon_phase = moon_phase
        self.uv_index = uv_index
        self.min_visibility = min_visibility
        self.pressure = pressure


class Weather:
    daily: List[Daily] = []
    hourly: List[Current] = []
    current: Current

    def __init__(self, daily: List[Daily], hourly: List[Current], current: Current) -> None:
        self.daily = []
        for param in daily:
            param = convertCamelCase(param)
            self.daily.append(Daily(**dict(param)))

        self.hourly = []
        for param in hourly:
            param = convertCamelCase(param)
            self.hourly.append(Current(**dict(param)))

        current = convertCamelCase(current)
        self.current = Current(**dict(current))


class WeatherData:
    data: Weather = []

    def __init__(self, data) -> None:
        self.data = Weather(**dict(data))

    def getDataOnTypeHourly(self, typeWeather):
        if typeWeather == 'time':
            return self.data.current.time
        elif typeWeather == 'temperature':
            return self.data.current.temperature
        elif typeWeather == 'feels_like_temp':
            return self.data.current.feels_like_temp
        elif typeWeather == 'rel_humidity':
            return self.data.current.rel_humidity
        elif typeWeather == 'dew_point':
            return self.data.current.dew_point
        elif typeWeather == 'wind_speed':
            return self.data.current.wind_speed
        elif typeWeather == 'wind_dir_string':
            return self.data.current.wind_dir_string
        elif typeWeather == 'wind_gust':
            return self.data.current.wind_gust
        elif typeWeather == 'precip_prob':
            return self.data.current.precip_prob
        elif typeWeather == 'precip_rate':
            return self.data.current.precip_rate
        elif typeWeather == 'cloudiness':
            return self.data.current.cloudiness
        elif typeWeather == 'thunder_prob':
            return self.data.current.thunder_prob
        elif typeWeather == 'uv_index':
            return self.data.current.uv_index
        elif typeWeather == 'pressure':
            return self.data.current.pressure
        elif typeWeather == 'visibility':
            return self.data.current.visibility
        else:
            return 0



from sdsRayanArvin.Dataset.AuxStationData import Aux
from sdsRayanArvin.RepositoryApi.DataApi import Data


class Pipeline:
    def GetLastData(self, Aux_data: Aux, deviceData: Data):
        if Aux_data is None:
            return 0

        type_aux = None

        if Aux_data.type == 1:
            type_aux = 'DO'
        if Aux_data.type == 2:
            type_aux = 'DI'
        if Aux_data.type == 3:
            type_aux = 'AI'
        if Aux_data.type == 5:
            type_aux = 'VDO'
        if Aux_data.type == 6:
            type_aux = 'AII'
        if Aux_data.type == 8:
            return self.ReplaceTagWithValue(Aux_data, deviceData)

        if type_aux is None:
            return

        Last_data = deviceData.LastDataOnTopic(str(Aux_data.code),
                                               str(type_aux),
                                               str(Aux_data.pan),
                                               str(Aux_data.address))

        if Last_data is not None:
            if Aux_data.type == 1 or Aux_data.type == 2 or Aux_data.type == 5:
                binOutput = "{0:b}".format(int(Last_data.getData()))
                binOutputArray = list(binOutput)
                binOutputArray.reverse()
                idxOutput = Aux_data.output - 1
                if len(binOutputArray) > idxOutput:
                    return binOutputArray[idxOutput]
                else:
                    return 0

            return Last_data.getData()
        else:
            return 0

    def ReplaceTagWithValue(self, Aux_data: Aux, deviceData: Data):
        arrayMath = Aux_data.Analog.math.split(' ')
        tag = arrayMath[0]
        Last_data = deviceData.LastDataOnTopic(str(Aux_data.code),
                                               str(tag),
                                               str(Aux_data.pan),
                                               str(Aux_data.address))
        data = Last_data.getData()

        data = 0 if data is None else data

        finalData = Aux_data.Analog.math.replace(tag, data)

        return self.ReplaceMathWithValue(finalData)

    def ReplaceMathWithValue(self, value):
        arrayMath = value.split(' ')
        value = 0

        for idx, val in enumerate(arrayMath):
            if idx == 0 or idx == len(arrayMath) - 1 or idx % 2 == 0:
                continue

            nextVal = float(arrayMath[idx + 1])
            preVal = float(arrayMath[idx - 1])

            if val == '+':
                if value == 0:
                    value = preVal + nextVal
                else:
                    value += nextVal

            if val == '-':
                if value == 0:
                    value = preVal - nextVal
                else:
                    value -= nextVal

            if val == '/':
                if value == 0:
                    value = preVal / nextVal
                else:
                    value /= nextVal

            if val == '*':
                if value == 0:
                    value = preVal * nextVal
                else:
                    value *= nextVal

        return value



from sdsRayanArvin.Dataset.UserData import UserData
from sdsRayanArvin.Repository import Repository


class Authenticate(Repository):
    def Login(self):
        res = self.Request.sendWithOutToken('POST',
                                            'user/authenticate',
                                            {
                                                'phone': '09100518330',
                                                'password': '51905190'
                                            })
        self.Request.setToken(res['token'])

    def UserInformation(self) -> UserData:
        """
        get user information
        :return: :class:`User <User>` object
        """

        res = self.Request.sendWithToken('GET', 'authenticated_user')
        return UserData(res['user'])



import threading


class Cache:
    def __init__(self):
        self.data = {}
        threading.Timer(10, self.clearCache).start()

    def setCache(self, name, data):
        self.data[name] = data

    def getCache(self, name):
        try:
            return self.data[name]
        except:
            return None

    def clearCache(self):
        self.data = {}



class Token:
    def __init__(self):
        self.__token = None

    def setToken(self, token):
        self.__token = token

    def getToken(self):
        return self.__token

    def hasToken(self):
        return self.__token is not None



from sdsRayanArvin.Dataset.ObjectData import ObjectData
from sdsRayanArvin.Request import Request
from sdsRayanArvin.Repository import Repository


class Object(Repository):
    def ObjectByIdOrg(self):
        res = self.Request.sendWithToken('GET', 'object_all')
        try:
            return ObjectData(res['Object']).data
        except:
            return []



from sdsRayanArvin.Repository import Repository


class Log(Repository):
    def SendLog(self, title, ID_workflows, value, ID_org, typeLog):
        res = self.Request.sendWithOutToken('POST', 'gelf', {
            "title": title,
            "type": typeLog,
            "org": ID_org,
            "short_message": ID_workflows,
            "value": value,
        }, 'json')

        return res



from sdsRayanArvin.Dataset import DeviceData
from sdsRayanArvin.Repository import Repository


class Data(Repository):
    def LastDataOnTopic(self, topic, type_data, pan, add):
        try:
            res = self.Request.sendWithToken('GET', 'data/last_by_type/' + topic + '/' + type_data + '/' + pan + '/' + add)
            return DeviceData(res['success'])
        except:
            return None

    def StoreDataOnTopic(self, data):
        res = self.Request.sendWithToken('POST', 'data', data)
        return res




from setuptools import setup

setup(
    name='sdsRayanArvin',
    version='0.97',
    description='System data server',
    author='vahid heydari',
    url='https://hamgit.ir/v.heidary13/sds',
    packages=[
        'sdsRayanArvin',
        'sdsRayanArvin.RepositoryApi',
        'sdsRayanArvin.RepositoryApi.OldApi',
        'sdsRayanArvin.RepositoryApi.DataApi',
        'sdsRayanArvin.RepositoryApi.WorkflowApi',
        'sdsRayanArvin.RepositoryApi.RuleApi',
        'sdsRayanArvin.RepositoryApi.WeatherApi',
        'sdsRayanArvin.RepositoryApi.ExeApi',
        'sdsRayanArvin.RepositoryApi.LogApi',
        'sdsRayanArvin.Dataset',
        'sdsRayanArvin.Library',
    ],
)



class UserData:
    def __init__(self, userData):
        self.__name = userData['name']
        self.__family = userData['family']
        self.__username = userData['username']
        self.__phone = userData['phone']
        self.__type = userData['type']
        self.__org = userData['org']

    def getName(self):
        return self.__name

    def getFamily(self):
        return self.__family

    def getUsername(self):
        return self.__username

    def getPhone(self):
        return self.__phone

    def getType(self):
        return self.__type

    def getIdOrg(self):
        return self.__org




from .UserData import UserData
from .ObjectData import ObjectData
from .DeviceData import DeviceData
from .WorkflowsData import WorkflowsData
from .RuleData import RuleData
from .AuxStationData import AuxStationData
from .WeatherData import WeatherData
